/*****************************************************************************
*               Copyright (c) Gemplus Card International 1997
*
* Name        : cardreader.xs
*
* Description : Perl Interfaces Definitions for Smart Card Readers
  Author      : Laurent GAUTERON, Ludovic ROUSSEAU
  Compiler    : 
  Host        : 
*
* $Id: cardreader.xs,v 1.3 2000/10/11 15:07:59 lgautero Exp $
* $Log: cardreader.xs,v $
* Revision 1.3  2000/10/11 15:07:59  lgautero
* Updating for Debian package modifications
*
* Revision 1.2  2000/09/04 08:19:08  rousseau
* code cleanup, changed the API and name for [Init|Close]Serial
*
*
* Last modif. : 20/01/2000 Laurent GAUTERON
*					Creation
*               03/05/2000 Ludovic ROUSSEAU
*					Object Oriented rewrite
*
******************************************************************************
*
* Warning     : 
*
* Remarks     : 
*
*****************************************************************************/


#ifdef __cplusplus
extern "C" {
#endif

#include <string.h>
#include <stdlib.h>
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "cardperl.h"

typedef enum
{
	SC_LOAD_LIBRARY = -1,
	SC_LOAD_FUNCTION = -2,
	SC_NOT_ENOUGH_MEMORY = -3,
	SC_CANT_OPEN_DEVICE = -4,
	SC_BAD_HANDLE = -5,
	SC_INVALID_TYPE = -6
} t_error;

#ifdef __cplusplus
}
#endif

MODULE = cardreader		PACKAGE = cardreader
PROTOTYPES: ENABLE


#####################################################
# _AllocReaderLib()									#
# First function to call.							#
# Returns FALSE if error else TRUE					#
# If an error occurs see $! 						#
#	-2 if the dynamic library can't be loaded  		#
#	-3 pointer if function can't be loaded			#
#####################################################
int
_AllocReaderLib()
	CODE:
		RETVAL = TRUE;

		/* load de dynamic library */
		if (hDll == NULL)
		{
			/* LOAD_LIB is an OS dependant macro defined in cardperl.h */
			hDll = LOAD_LIB;

			if (hDll == NULL)
			{
				errno = SC_LOAD_LIBRARY;
				RETVAL = FALSE;
				sv_setpv(perl_get_sv("!", 1),
					"dynamic library can't be loaded");
			}
			else
			{
				/* load the pointers of function */
				pInitReader = 
					(TInitReader) GET_FCT(hDll, "InitReader");
				
				pCloseReader = 
					(TCloseReader) GET_FCT(hDll, "CloseReader");

				pResetCard =
					(TResetCardAscii) GET_FCT(hDll, "ResetCardAscii");
				
				pSendIncomingAPDU = 
					(TSendIncomingAPDUAscii) GET_FCT(hDll,
					"SendIncomingAPDUAscii");

				pSendOutgoingAPDU = 
					(TSendOutgoingAPDUAscii) GET_FCT(hDll,
					"SendOutgoingAPDUAscii");

				pPowerDown = (TPowerDown) GET_FCT(hDll, "PowerDown");
				
				pSendIncomingOutgoingAPDU = 
					(TSendIncomingOutgoingAPDUAscii) 
					GET_FCT(hDll, "SendIncomingOutgoingAPDUAscii");
				
				pGetRerrno = (TGetRerrno) GET_FCT(hDll, "GetRerrno");

				pGetRerrStr = (TGetRerrStr) GET_FCT(hDll, "GetRerrStr");

				pISO7816_ErrorStr = (TISO7816_ErrorStr) GET_FCT(hDll,
					"ISO7816_ErrorStr");

				if (!pInitReader || !pCloseReader || !pResetCard ||
					!pSendIncomingAPDU || !pSendOutgoingAPDU ||
					!pPowerDown || !pSendIncomingOutgoingAPDU ||
					!pGetRerrno || !pGetRerrStr || !pISO7816_ErrorStr)
				{
					errno = SC_LOAD_FUNCTION;
					RETVAL = FALSE;
					sv_setpv (perl_get_sv("!",1),
						"function(s) can't be found in the dynamic library");
				}
			}
		}
		
	OUTPUT:
		RETVAL


#####################################################
# _InitReader()										#
# Return a handle of the reader of type 'type'		#
# specified by 'device'.							#
# return -1 if an error occurs.						#
# If an error occurs :								# 
# $! = -1 when the serial port can't be open.		#
# $! = -2 if a cardreader handle can't be allocated.#
# otherwise return the index corresponding			#
# to the ReaderDesc									#
#####################################################
ReaderDesc *
_InitReader(type, device)
int type
char* device
	CODE:
		int l_iRet;
		ReaderDesc *p_iDesc;
				
		/* Allocate a new cardreader */
		p_iDesc = (ReaderDesc *)malloc(sizeof(ReaderDesc));
		
		if (p_iDesc == NULL)
		{
			errno = SC_NOT_ENOUGH_MEMORY;
			RETVAL = NULL;
			sv_setpv (perl_get_sv("!",1),
				"not enough memory");
		}
		else
		{
			l_iRet = pInitReader(p_iDesc, (rd_type)type, device);
			if (l_iRet < 0)
			{
				/* the new cardreader allocated can't be opened */
				sv_setpv (perl_get_sv("!", 1),
					"the device can't be opened (it could already be opened)");
				
				errno = SC_CANT_OPEN_DEVICE;
				RETVAL = NULL;
			}
			else
				RETVAL = p_iDesc;
		}

	OUTPUT:
		RETVAL


#####################################################
#	_CloseReader ()									#
#	Close the communication with the reader			#
#	and free the handle.							#
#	Return TRUE if the function succeeds else FALSE.#
#	If the function fails :							#
#	$! = -1 for an invalid handle					#
#####################################################
int        
_CloseReader(p_iDesc)
ReaderDesc *p_iDesc
	CODE:
		if (p_iDesc)
		{
			pCloseReader(p_iDesc);
			RETVAL = TRUE;
		}
		else
		{
			RETVAL = FALSE;
			errno = SC_BAD_HANDLE;
			sv_setpv (perl_get_sv("!", 1), "Invalid handle");
		}

	OUTPUT:
		RETVAL


#####################################################
#	_PowerDown()									#
#	Sends a power down to the specified card reader	#
#	return TRUE if OK else FALSE.					#
#	If the function fails 	:						#
#	$! = -1 for an invalid reader handle			#
#	$! = errcode									#
#####################################################
int
_PowerDown(p_iDesc)
ReaderDesc *p_iDesc
	CODE:
		if (p_iDesc)
		{
			RETVAL = pPowerDown(p_iDesc);
			if (RETVAL < 0)
			{
				RETVAL = FALSE;
				errno = pGetRerrno();
				sv_setpv (perl_get_sv("!", 1), pGetRerrStr(errno));
			}
			else
			{
				RETVAL = TRUE;
			}
		}
		else
		{
			RETVAL = FALSE;
			errno = SC_BAD_HANDLE;
			sv_setpv (perl_get_sv("!", 1), "Invalid handle");
		}

	OUTPUT:
		RETVAL


#####################################################
#	_ResetCard()									#
#	Sends a reset to the card.						#
#	A valid reader handle must be passed.			#
#	Returns an ATR as a string						#
#	If an error occurs an empty string is returned,	#
#	If an error occurs :							#
#	$! = -1 for an invalid handle					#
#	$! = errcode									#
#####################################################
void
_ResetCard(p_iDesc)
ReaderDesc *p_iDesc
	PPCODE:
		memset (szCharBuffer, 0, sizeof(szCharBuffer));
		if (p_iDesc)
		{
			pResetCard(p_iDesc, szCharBuffer, sizeof(szCharBuffer));
			errno = pGetRerrno();
			sv_setpv (perl_get_sv("!", 1), pGetRerrStr(errno));
		}
		else
		{
			errno = SC_BAD_HANDLE;
			sv_setpv (perl_get_sv("!", 1), "Invalid handle");
		}
		
		# return the result (string) on the stack
		XPUSHs (sv_2mortal(newSVpv(szCharBuffer, 0)));
		XSRETURN(1);


#####################################################
#	_SendIncomingAPDU()								#
#	Sends an incomming APDU to the card.			#
#	A valid reader handle must be passed.			#
#	Returns the status word as a string : SW1SW2	# 	
#	If an error occurs an empty string is returned,	#
#	If an error occurs :							#
#	$! = -1 for an invalid handle					#
#	$! = errcode									#
#####################################################
void 
_SendIncomingAPDU(p_iDesc, APDU_Data)
ReaderDesc *p_iDesc
char* APDU_Data
	PPCODE:
		memset (szCharBuffer, 0, sizeof(szCharBuffer));
		if (p_iDesc)
		{
			pSendIncomingAPDU(p_iDesc,
				APDU_Data, szCharBuffer, sizeof(szCharBuffer));
			errno = pGetRerrno();
			sv_setpv (perl_get_sv("!", 1), pGetRerrStr(errno));
		}
		else
		{
			errno = SC_BAD_HANDLE;
			sv_setpv (perl_get_sv("!", 1), "invalid handle");
		}
		
		#return the result (string) on the stack
		XPUSHs (sv_2mortal(newSVpv(szCharBuffer, 0)));
		XSRETURN(1);
	
		
#####################################################
#	_SendOutgoingAPDU()								#
#	Send an out going APDU to the card.				#
#	A valid reader handle must be passed.			#
#	Returns 2 strings : the first one is the data,	#
#	the second is the status word.					#
#	If an error occurs, the first and/or the second #
#	string can be empty.							#
#	$! = -1 for an invalid handle					#
#	$! = errcode									#
#####################################################
void 
_SendOutgoingAPDU(p_iDesc, APDU_Data)
ReaderDesc *p_iDesc
char* APDU_Data
	PPCODE:
		memset (szCharBuffer, 0, sizeof(szCharBuffer));
		if (p_iDesc)
		{
			pSendOutgoingAPDU(p_iDesc,
				APDU_Data, szCharBuffer, sizeof(szCharBuffer));
			errno = pGetRerrno();
			sv_setpv (perl_get_sv("!", 1), pGetRerrStr(errno));
		}
		else
		{
			errno = SC_BAD_HANDLE;
			sv_setpv (perl_get_sv("!", 1),
				"invalid handle");
		}
		
		/* return the result (array of string) on the stack
		The first elem of the array is the DATA, the second
		is the status word
		*/
		if (strlen(szCharBuffer) > 4)
		{
			XPUSHs (sv_2mortal(newSVpv(szCharBuffer, strlen(szCharBuffer)-4)));
			XPUSHs (sv_2mortal(newSVpv(szCharBuffer+strlen(szCharBuffer)-4, 4)));
		}
		else
			if (strlen(szCharBuffer) == 4)
			{
				XPUSHs (sv_2mortal(newSVpv("", 0)));
				XPUSHs (sv_2mortal(newSVpv(szCharBuffer, 0)));
			}
			else
			{
				XPUSHs (sv_2mortal(newSVpv("", 0)));
				XPUSHs (sv_2mortal(newSVpv("", 0)));
			}
		XSRETURN(2);


#####################################################
#	_SendIncomingOutgoingAPDU()						#
#	Send an incoming data to the card and get the	#
#	response.										#
#	A valid reader handle must be passed.			#
#	Returns 2 strings : the first one is the data,	#
#	the second is the status word.					#
#	If an error occurs, the first and/or the second #
#	string can be empty.							#
#	$! = -1 for an invalid handle					#
#	$! = errcode									#
#####################################################
void 
_SendIncomingOutgoingAPDU(p_iDesc, APDU_Data)
ReaderDesc *p_iDesc
char* APDU_Data
	PPCODE:
		memset (szCharBuffer, 0, sizeof(szCharBuffer));
		if (p_iDesc)
		{
			pSendIncomingOutgoingAPDU(p_iDesc,
				APDU_Data, szCharBuffer, sizeof(szCharBuffer));
			errno = pGetRerrno();
			sv_setpv (perl_get_sv("!", 1), pGetRerrStr(errno));
		}
		else
		{
			errno = SC_BAD_HANDLE;
			sv_setpv (perl_get_sv("!", 1),
				"invalid handle");
		}

		/* return the result (array of string) on the stack
		The first elem of the array is the DATA, the second
		is the status word
		*/
		if (strlen(szCharBuffer) > 4)
		{
			XPUSHs (sv_2mortal(newSVpv(szCharBuffer, strlen(szCharBuffer)-4)));
			XPUSHs (sv_2mortal(newSVpv(szCharBuffer+strlen(szCharBuffer)-4, 4)));
		}
		else
			if (strlen(szCharBuffer) == 4)
			{
				XPUSHs (sv_2mortal(newSVpv("", 0)));
				XPUSHs (sv_2mortal(newSVpv(szCharBuffer, 0)));
			}
			else
			{
				XPUSHs (sv_2mortal(newSVpv("", 0)));
				XPUSHs (sv_2mortal(newSVpv("", 0)));
			}

		XSRETURN(2);


#####################################################
#	_ISO7816_ErrorStr()								#
#	return the ISO 7816 text error					#
#####################################################
void
_ISO7816_ErrorStr(code)
char *code
	PPCODE:
		int sw1, sw2;
		char *ptr;

		# get the status words
		sscanf(code, "%x", &sw1);
		sw2 = sw1 & 0xFF;
		sw1 >>= 8;

		ptr = pISO7816_ErrorStr(sw1, sw2);

		# return the result (string) on the stack
		XPUSHs (sv_2mortal(newSVpv(ptr, 0)));
		XSRETURN(1);


MODULE = cardreader		PACKAGE = ReaderDescPtr

#####################################################
#	DESTROY											#
#	automatic destruction of the object				#
#####################################################
void
DESTROY(p_iDesc)
ReaderDesc *p_iDesc
	CODE:
#		fprintf(stderr, "pr-libration %p\n", p_iDesc);
		if (p_iDesc)
		{
			pCloseReader(p_iDesc);

#			fprintf(stderr, "libration %p\n", p_iDesc);
			free(p_iDesc);
			p_iDesc = NULL;
		}

