#!/usr/bin/perl -w

##################################
# Perl script to analyse ATR
# By Christophe LEVANTIS
# 30 aug 2000
#
# $Id: ATR_analyse.pl,v 1.1 2000/09/22 16:00:17 rousseau Exp $
# 
# $Log: ATR_analyse.pl,v $
# Revision 1.1  2000/09/22 16:00:17  rousseau
# Analyse the ATR
#
#
##################################
#use integer;
use strict;
use cardreader;

my (%TS, @Fi, @Di, @XI, @UI, $T, $value, $tabcd, $counter, $line,
	@object, $nb_bytes);

my ($scr, $sw, $atr);

%TS = (0x3B, "Direct Convention", 0x3F, "Inverse Convention");
@Fi = (372, 372, 558, 744, 1116, 1488, 1860, "RFU", "RFU", 512, 768, 1024,
	1536, 2048, "RFU", "RFU");
@Di = ("RFU", 1, 2, 4, 8, 16, 32, "RFU", 12, 20, "RFU", "RFU", "RFU",
	"RFU", "RFU", "RFU");

@XI = ("not supported", "state L", "state H", "no preference");
@UI = ("A only (5V)", "B only (3V)", "A and B", "RFU");

$T=0;

$value=0;
$tabcd=0;
$counter=1;

# 1_ 1 argument then input = ATR else smart card
if ($#ARGV == 0)
{
	$atr = $ARGV[0];
}
else
{
	######################   new    ####################
	# allocate a descriptor
	# the default device is /dev/screader
	$scr = new cardreader or die "Can't allocate reader: $!\n";

	######################   ResetCard   ####################
	$atr = $scr -> ResetCard;

	###############   PowerDown  ################
	$scr -> PowerDown or die "PowerDown: $!\n";

	undef $scr;
}

print "ATR: $atr\n";



# 2_ Split in bytes of the lines
$atr =~ s/(..)/$1 /g;
@object = split(/\s/, $atr);
$nb_bytes=$#object+1;
#print $nb_bytes;

# 3_ Analysis

# Analysis of TS :
$value=hex(shift(@object));
printf ("+ TS = %X --> %s\n", $value, $TS{$value});

# Analysis of T0 :
$value=hex(shift(@object)); $tabcd=$value >> 4;
$value=$value % 16;
printf ("+ T0 = %d --> octets historiques\n\n", $value);

print "-----\n";
if ($tabcd & 0x1) { analyse_TA() };
if ($tabcd & 0x2) { analyse_TB() };
if ($tabcd & 0x4) { analyse_TC() };
if ($tabcd & 0x8) { analyse_TD() };

print "\n\n";
#Il ne reste plus que les octets historiques
print "+ Octets historiques : @object\n";

######## Sub functions
sub analyse_TA   {
# @_ contient les arguments

	 $value=hex(shift(@object));
	 printf ("  TA($counter) present %02X --> ", $value);

	 # TA1 Analysis
	 if ($counter==1) { 
	    my $F=$value >> 4; my $D=$value % 16;
	    $value=$Fi[$F]/$Di[$D];
	    printf ("Fi=%s, Di=%s %.3f c/etu", $Fi[$F], $Di[$D],$value);
	 }
	 
	 # TA2 Analysis - TA2 is the specific mode byte
	 if ($counter==2) { 
	    my $F=$value >> 4; my $D=$value % 16;
	    printf ("Protocol to be used in spec mode : T=%s", $D);
	    if ($F & 0x8) { print " - Unable to change"; }
	    else             { print " - Capable to change"; }

	    if ($F & 0x1) { print " - implicity defined"; }
	    else             { print " - defined by interface bytes"; }
	 }   

	 # TA3 Analysis
	 if ($counter>=3) {

	    if ($T==1) {
	      printf ("IFSC : %s", $value);
	    }
	    else  {         #### T <> 1
	      my $F=$value >> 6; my $D=$value % 64;
	      my $Class="(3G) ";
	      if ($D & 0x1) { $Class=$Class."A 5V " };
	      if ($D & 0x2) { $Class=$Class."B 3V " };
	      if ($D & 0x4) { $Class=$Class."C 1.8V " };
	      if ($D & 0x8) { $Class=$Class."D RFU " };
	      if ($D & 0x10) { $Class=$Class."E RFU" };
	      printf ("Clock stop : %s - Class accepted by the card : %s", $XI[$F],$Class); 
	    }
	 }
	 print "\n";
}

sub analyse_TB   {
	 $value=hex(shift(@object));
	 printf ("  TB($counter) present %02X --> ", $value);


	 my $I=$value >> 5; my $PI=$value % 32;

	 if ($counter==1) {
	    print "Prog Param P : $PI, I : $I";
	 }

	 if ($counter==2) {
	    print "Prog param PI2 (PI1 sould be ignored) : ";
	    if (($value>49)||($value<251)) { print "$value (dV)"; }
	    else { print "$value is RFU"; }
	 }

	 if ($counter>=3) {
	    if ($T==1) {
	       my $BWI=$value >> 4; my $CWI=$value % 16;
	       printf ("Block Wainting Integer : %s - Character Waiting Integer : %s", $BWI, $CWI);
	    }
	 }
	 print "\n";
}

sub analyse_TC   {
	 $value=hex(shift(@object));
	 printf ("  TC($counter) present %02X --> ", $value);


	 if ($counter==1) {
	    print "Extra guard time : $value";
	 }

	 if ($counter==2) {
	    printf ("Work waiting time : 960 x %d x (Fi/F)", $value);
	 }

	 if ($counter>=3) {
	    if ($T==1) {
	       printf ("Error detection code : ");
	       if ($value==1) { print "CRC"; }
	       elsif ($value==0) { print "LRC"; }
	       else { print "RFU"; }
	    }
	 }
	 print "\n";
}

sub analyse_TD   {
	 my $str='';
	 $value=hex(shift(@object));
	 my $Y=$value >> 4; $T=$value % 16;
	 if ($T==15) { $str=" - Global interface bytes following"; }
	 printf ("  TD($counter) present %02X --> T = $T$str\n", $value);

	 $counter++;
	 print "-----\n";
	 if ($Y & 0x1) { analyse_TA() };
	 if ($Y & 0x2) { analyse_TB() };
	 if ($Y & 0x4) { analyse_TC() };
	 if ($Y & 0x8) { analyse_TD() };
}


