#!/usr/bin/perl -w

#
#	sc_shell.pl - small smart card I/O interpreter
#	Ludovic Rousseau
#	dim sep  3 17:44:01 CEST 2000
#

use strict;


#####################################################################
#																	#
#						small shell interpreter						#
#																	#
#####################################################################
sub sc_shell
{
	my $scr = shift;
	my $in = shift;
	my $out = shift;
	my $echo = shift;
	my $line;
	my ($ret, $data);

	while (1)
	{
		$line = <$in>;
		return if (! defined $line);

		print "=>$line" if ($echo);
		chomp $line;

		# R - RESET
		if ($line =~ m/^R/)
		{
			$ret = $scr -> ResetCard;
			if ($ret ne "")
			{
				print $out "<=$ret\n";
			}
			else
			{
				print $out "ResetCard error: $!\n";
			}
		}
		# Q - QUIT
		elsif ($line =~ m/^Q/)
		{
			print $out "Bye\n";
			close ($out);
			return;
		}
		else
		{
			# P - POWER OFF
			if ($line =~ m/^P/)
			{
				$scr -> PowerDown or die "PowerDown: $!\n";

				print $out "PowerDown\n";
			}
			# I - SendIncomingAPDU
			elsif ($line =~ m/^I/)
			{
				my ($cmd, $sw);

				$cmd = substr $line, 1;
				if ($sw = $scr -> SendIncomingAPDU($cmd))
				{
					print $out "sw $sw, ", $scr -> ISO7816_ErrorStr($sw), "\n";
				}
				else
				{
					print $out "SendIncomingAPDU: $!\n";
				}
			}
			# O - SendOutgoingAPDU
			elsif ($line =~ m/^O/)
			{
				my ($cmd, $sw);

				$cmd = substr $line, 1;
				if (($data, $sw) = $scr -> SendOutgoingAPDU($cmd))
				{
					print $out "<=$data\n";
					print $out "  >", hex2ascii($data), "<\n";
					print $out "sw $sw, ", $scr -> ISO7816_ErrorStr($sw), "\n";
				}
				else
				{
					print $out "SendOutgoingAPDU: $!\n";
				}
			}
			# X - SendIncomingOutgoingAPDU
			elsif ($line =~ m/^X/)
			{
				my ($cmd, $sw);

				$cmd = substr $line, 1;
				if (($data, $sw) = $scr -> SendIncomingOutgoingAPDU($cmd))
				{
					print $out "<=$data\n";
					print $out "  ", hex2ascii($data), "\n";
					print $out "sw $sw, ", $scr -> ISO7816_ErrorStr($sw), "\n";
				}
				else
				{
					print $out "SendIncomingOutgoingAPDU: $!\n";
				}
			}
			else
			{
				# une erreur ?
				print $out "Command unknown, $line\n";
			}
		}
	}
}

sub hex2ascii
{
	my $in = shift;
	my $out = "";
	my $v;

	$in =~ s/(..)/$1 /g;
	for (split ' ', $in)
	{
		$v = hex;
		if ($v > 31)
		{
			$out .= chr $v;
		}
		else
		{
			$out .= ".";
		}
	}

	return $out;
}

1;

