#!/usr/bin/perl -w

# 
#   testgpk.pl - test a GPK sign operation
#
#   Copyright (C) 1999 Ludovic Rousseau
#	lun avr 24 20:08:56 CEST 2000
#


use strict;
use cardreader;
# use MD5;
use Digest::MD5  qw(md5 md5_hex md5_base64);
use Math::BigInt;

my ($device, $scd, $atr, $cmd, $apdu);
my ($msg, $sig);
my ($n, $p, $q, $v, $s, $verif, $prevu);
my (@tab);

# message to sign
$msg = "00000000000000000000000000000000";

# n
$n = Math::BigInt->new(hex2dec("c5cabb0e65bc96c82d257cd123b9ef9bacc6bb028b76c2abe94b4f768ce0128361ec2c59b67bc0f7b7e2487f1f77cb883e3fc1d0142ba7009ffdc76c72579681"));

# p
$p = Math::BigInt->new(hex2dec("cf6957382179f5c1bbaf9549b870c680d6521230d8aed4b2d8eda1cde571e58f"));

# q
$q = Math::BigInt->new(hex2dec("f4207ed479c26a802140a00f5c0006dbc45cd96f74d22063f07bae536dd25aef"));

# v
$v = Math::BigInt->new(hex("0x10001"));

# s
#$s = PowerMod[v,-1, (p-1)(q-1)];
$s = Math::BigInt->new(5883234579855391732973053182113878969092811973231285691122425685805052872186314676692635104578430819352272601558812240472863444435915471431241589455117713);

$scd = new cardreader or die "new: $!\n";

# Reset card
$atr = $scd -> ResetCard or die "ResetCard: $!\n";
print STDERR "ATR: $atr\n";

# Select MF
$apdu = "00"."A4"."00"."0C"."02"."3F00";
print "apdu: $apdu\n";
$cmd = $scd -> SendIncomingAPDU($apdu);
print "sw $cmd ", $scd -> ISO7816_ErrorStr($cmd), "\n";

# Vrification du PIN code qui protge l'utilisation de la cl
$apdu = "00"."20"."00"."00"."08"."0000000000000000";
print "apdu: $apdu\n";
$cmd = $scd -> SendIncomingAPDU($apdu);
print "sw $cmd ", $scd -> ISO7816_ErrorStr($cmd), "\n";

# Select Crypto Context : Sign MD5 / RSA 512
$cmd = $scd -> SendIncomingAPDU("80"."A6"."03"."11"."00");
print "sw $cmd ", $scd -> ISO7816_ErrorStr($cmd), "\n";

# Put Crypto Data - Data en TLV
$apdu = "80"."DA"."11".sprintf("%02X", length($msg)/2).sprintf("%02X",
	length($msg)/2+2)."55".sprintf("%02X", length($msg)/2).$msg;
print "apdu: $apdu\n";
$cmd = $scd -> SendIncomingAPDU($apdu);
print "sw $cmd ", $scd -> ISO7816_ErrorStr($cmd), "\n";

# PK Sign
$apdu = "80"."86"."00"."00"."40";
print "apdu: $apdu\n";
($sig, $cmd) = $scd -> SendOutgoingAPDU($apdu);

print "sw $cmd ", $scd -> ISO7816_ErrorStr($cmd), "\n";
print "sig: $sig\n";

# verification
$sig =~ s/(..)/$1 /g;
$sig = join ('', reverse split (' ', $sig));
$verif = Math::BigInt->new(hex2dec($sig));
$verif = powermod($verif, $v, $n);
print "verif: $verif\n";

# prvu
$prevu = hex2dec(PadGPKPKSignRSAMD5($msg));
print "prevu: $prevu\n";

# comparaison
if ($prevu == $verif)
{
	print "OK\n";
}
else
{
	print "NOT OK\n";
}

# power down card
$scd -> PowerDown;

# close serial port
undef $scd;

#
# Padding RSA MD5 for GPK
#
sub	PadGPKPKSignRSAMD5
{
	my $msg = shift;
	my $ret;

	$ret = "00"."01"."FF" x (64 - 16 - 18 -3) . "00" .
		"3020300C06082A864886F70D020505000410" . md5_hex(hex2bin($msg));

	return $ret;
}

#
# modular exponentiation
# powermod (a, b, c) = a^b mod c
#
sub powermod
{
	my $m = Math::BigInt->new(shift);
	my $e = Math::BigInt->new(shift);
	my $n = Math::BigInt->new(shift);
	my ($r, $p);

	return 1 if ($e == 0);

	$r = Math::BigInt->new(1);

	# carr-multipli
	while ($e != 0)
	{
		if ($e % 2 != 0)
		{
			$r = ($r * $m) % $n;
		}

		$m = ($m * $m) % $n;

		# bit suivant
		$e = $e / 2;
	}

	return $r;
}

#
# convert from a hexa strings to a binary string
#
sub hex2bin
{
	my $msg = shift;
	my $ret;

	$msg =~ s/(..)/$1 /g;
	for (split ' ', $msg)
	{
		$ret .= chr hex;
	}

	return $ret;
}

#
# convert an hexadecimal big number into decimal
#
sub hex2dec
{
	my $n = shift;
	my $ret;

	$ret = Math::BigInt->new(0);
	$n =~ s/(..)/$1 /g;
	for (split ' ', $n)
	{
		$ret = $ret*256 + hex;
	}

	return $ret;
}

